@@ -37,7 +37,7 @@ my %aliases = (
     cache_it        => 'cache_it',
     read_script     => 'read_script',
     rewrite_shebang => 'rewrite_shebang',
-    set_script_name => 'set_script_name',
+    get_script_name => 'get_script_name',
     chdir_file      => 'chdir_file_normal',
     get_mark_line   => 'get_mark_line',
     compile         => 'compile',
