@@ -14,7 +14,7 @@
 #define modperl_envelem_tie(sv, key, klen) \
     sv_magic(sv, Nullsv, 'e', key, klen)
 
-void modperl_env_hash_keys(void);
+void modperl_env_hash_keys(pTHX);
 
 void modperl_env_clear(pTHX);
 
@@ -26,6 +26,8 @@ void modperl_env_default_populate(pTHX);
 
 void modperl_env_request_populate(pTHX_ request_rec *r);
 
+void modperl_env_request_unpopulate(pTHX_ request_rec *r);
+
 void modperl_env_request_tie(pTHX_ request_rec *r);
 
 void modperl_env_request_untie(pTHX_ request_rec *r);
