@@ -572,7 +572,7 @@ MP_INLINE SV *modperl_dir_config(pTHX_ request_rec *r, server_rec *s,
                                        key, sv_val, FALSE);
     }
 
-    if (!SvTRUE(retval)) {
+    if (!SvOK(retval)) {
         if (s && s->module_config) {
             MP_dSCFG(s);
             SvREFCNT_dec(retval); /* in case above did newSV(0) */
@@ -706,13 +706,21 @@ MP_INLINE SV *modperl_slurp_filename(pTHX_ request_rec *r, int tainted)
     
     size = r->finfo.size;
     sv = newSV(size);
-    file = r->finfo.filehand;
-    if (!file) {
-        rc = apr_file_open(&file, r->filename, APR_READ|APR_BINARY,
-                           APR_OS_DEFAULT, r->pool);
-        SLURP_SUCCESS("opening");
+
+    if (!size) {
+        sv_setpvn(sv, "", 0);
+        return newRV_noinc(sv);
     }
 
+    /* XXX: could have checked whether r->finfo.filehand is valid and
+     * save the apr_file_open call, but apache gives us no API to
+     * check whether filehand is valid. we can't test whether it's
+     * NULL or not, as it may contain garbagea
+     */
+    rc = apr_file_open(&file, r->filename, APR_READ|APR_BINARY,
+                       APR_OS_DEFAULT, r->pool);
+    SLURP_SUCCESS("opening");
+
     rc = apr_file_read(file, SvPVX(sv), &size);
     SLURP_SUCCESS("reading");
 
@@ -761,3 +769,53 @@ void modperl_apr_table_dump(pTHX_ apr_table_t *table, char *name)
     }    
 }
 #endif
+
+#define MP_VALID_PKG_CHAR(c) (isalnum(c) ||(c) == '_')
+#define MP_VALID_PATH_DELIM(c) ((c) == '/' || (c) =='\\')
+char *modperl_file2package(apr_pool_t *p, const char *file)
+{
+    char *package;
+    char *c;
+    const char *f;
+    int len = strlen(file)+1;
+
+    /* First, skip invalid prefix characters */
+    while (!MP_VALID_PKG_CHAR(*file)) {
+        file++;
+        len--;
+    }
+
+    /* Then figure out how big the package name will be like */
+    for (f = file; *f; f++) {
+        if (MP_VALID_PATH_DELIM(*f)) {
+            len++;
+        }
+    }
+
+    package = apr_pcalloc(p, len);
+
+    /* Then, replace bad characters with '_' */
+    for (c = package; *file; c++, file++) {
+        if (MP_VALID_PKG_CHAR(*file)) {
+            *c = *file;
+        }
+        else if (MP_VALID_PATH_DELIM(*file)) {
+
+            /* Eliminate subsequent duplicate path delim */
+            while (*(file+1) && MP_VALID_PATH_DELIM(*(file+1))) {
+                file++;
+            }
+ 
+            /* path delim not until end of line */
+            if (*(file+1)) {
+                *c = *(c+1) = ':';
+                c++;
+            }
+        }
+        else {
+            *c = '_';
+        }
+    }
+   
+    return package;
+}
