@@ -15,7 +15,7 @@ my $pool = Apache->server->process->pool;
 my $project_root = canonpath Apache::server_root_relative($pool, "..");
 my (@a, @b, @c);
 for (@INC) {
-    if (m|^$project_root|) {
+    if (m|^\Q$project_root\E|) {
         m|blib| ? push @b, $_ : push @a, $_;
     }
     else {
@@ -88,7 +88,7 @@ Apache->server->add_config(['<Perl >', '1;', '</Perl>']);
 # are running on). This should work starting from perl-5.8.1 and higher.
 use Config;
 if ($] >= 5.008001 && $Config{useithreads}) {
-    eval { require threads; threads->import() };
+    eval { require threads; "threads"->import() };
 }
 
 
@@ -162,6 +162,9 @@ sub ModPerl::Test::exit_handler {
     my($p, $s) = @_;
 
     $s->log->info("Child process pid=$$ is exiting");
+
+    Apache::OK;
+
 }
 
 END {
