@@ -3,6 +3,12 @@ package TestAPI::sendfile;
 use strict;
 use warnings FATAL => 'all';
 
+use Apache::RequestRec ();
+use Apache::RequestIO ();
+
+use APR::Const -compile => 'SUCCESS';
+use Apache::Const -compile => ':common';
+
 sub handler {
     my $r = shift;
 
