@@ -83,8 +83,10 @@ arg = *MARK++
 #define mpxs_write_loop(func, obj) \
     while (MARK <= SP) { \
         apr_ssize_t wlen; \
+        apr_status_t rv; \
         char *buf = SvPV(*MARK, wlen); \
-        apr_status_t rv = func(aTHX_ obj, buf, &wlen); \
+        MP_TRACE_o(MP_FUNC, "%d bytes [%s]", wlen, buf); \
+        rv = func(aTHX_ obj, buf, &wlen); \
         if (rv != APR_SUCCESS) { \
             Perl_croak(aTHX_ modperl_apr_strerror(rv)); \
         } \
